{$APPTYPE        GUI}

{$IFDEF FPC}
  {$MODE       Delphi}
  {$MODESWITCH ANSISTRINGS-}
{$ENDIF}

{$LONGSTRINGS    OFF}
{$WRITEABLECONST ON}


{$R textout.RES}

program _TextOut;

uses Windows, Messages, Resource;

const
  AppName    = 'TextOut Example';
  AboutBox   = 'AboutBox';
  APPICON    = 'APPICON';
  APPMENU    = 'APPMENU';

function About(DlgWnd : hWnd; Msg, wParam, lParam : DWORD) : BOOL; stdcall;
begin
  About := TRUE;

  case msg of

    WM_INITDIALOG: exit;

    WM_COMMAND:
       begin
         if (LOWORD(wParam) = IDOK) or (LOWORD(wParam) = IDCANCEL) then
           begin
             EndDialog(DlgWnd, Ord(TRUE));
             exit;
           end;
       end;
  end;

  About := FALSE;
end;

function WndProc (Wnd : hWnd; Msg, wParam, lParam : DWORD) : DWORD; stdcall;
  { main application/window handler function                                  }
const
  TextOut_CallA = 'TextOut (dc : HDC; x, y : integer; Text : pchar; ';
  TextOut_CallB = 'TextLen : integer) : BOOL;';

  Text  = 'TextOut, just a simple example.';
var
  ps                 : TPAINTSTRUCT;

  ClientRect         : TRECT;

  Buf                : packed array[0..255] of char;

  TextSize           : TSIZE;
begin
  WndProc := 0;

  case Msg of
    WM_PAINT:
       begin
         BeginPaint(Wnd, ps);

         { set up the dc                                                      }

         GetClientRect(Wnd, ClientRect);       { we'll use this quite often   }
         SetBkMode(ps.hdc, TRANSPARENT);
         SetTextAlign(ps.hdc, TA_CENTER or TA_BOTTOM);

         TextOut(ps.hdc,
                 ClientRect.Right  div 2,
                 ClientRect.Bottom div 2,
                 Text,
                 lstrlen(Text));

         {--------------------------------------------------------------------}
         { draw the function call                                             }

         SelectObject(ps.hdc, GetStockObject(DEFAULT_GUI_FONT));

         lstrcpy(Buf, TextOut_CallA);
         lstrcat(Buf, TextOut_CallB);

         { calculate the size of the output string                            }

         GetTextExtentPoint32(ps.hdc,
                              Buf,
                              lstrlen(Buf), TextSize);
         TextOut(ps.hdc,
                 ClientRect.Right div 2,
                 ClientRect.Bottom - TextSize.cy,
                 Buf,
                 lstrlen(Buf));

         {--------------------------------------------------------------------}
         { we're done painting                                                }

         EndPaint(Wnd, ps);

         exit;
       end;

    WM_COMMAND:
       begin
         case LOWORD(wParam) of
           IDM_ABOUT:
               begin
                 DialogBox(hInstance, ABOUTBOX, Wnd, @About);
                 exit;
               end; { IDM_ABOUT }

           IDM_EXIT:
               begin
                 DestroyWindow(Wnd);
                 exit;
               end; { IDM_EXIT }
         end; { case LOWORD(wParam) }
       end; { WM_COMMAND }

    WM_DESTROY:
       begin
         { DefWindowProc doesn't call PostQuitMessage.  If this API is not    }
         { called then the program will disappear but its EXE is still used   }
         { by Windows, this creates all kinds of unexpected problems.         }

         PostQuitMessage(0);
         exit;
       end; { WM_DESTROY }
  end; { case msg }

  WndProc := DefWindowProc (Wnd, Msg, wParam, lParam);
end;


function InitAppClass: WordBool;
  { registers the application's window classes                                }
var
  cls : TWndClassEx;

begin
  cls.cbSize            := sizeof(TWndClassEx);         { must be initialized }

  if not GetClassInfoEx (hInstance, AppName, cls) then
    begin
      with cls do begin
        { cbSize has already been initialized as required above               }

        style           := CS_BYTEALIGNCLIENT;
        lpfnWndProc     := @WndProc;                    { window class handler}
        cbClsExtra      := 0;
        cbWndExtra      := 0;

        // {$IFDEF VER90}                                  { Delphi 2.0          }
        hInstance       := system.hInstance;            { qualify instance!   }
        {// $ENDIF}

        // {$IFDEF FPC}
        // hInstance       := GetModuleHandle(nil);
        // {$ENDIF}

        hIcon           := LoadIcon (hInstance, APPICON);
        hCursor         := LoadCursor(0, idc_arrow);
        hbrBackground   := COLOR_WINDOW + 1;
        lpszMenuName    := APPMENU;                     { Menu name           }
        lpszClassName   := AppName;                     { Window Class name   }
        hIconSm         := LoadImage(hInstance,
                                     APPICON,
                                     IMAGE_ICON,
                                     16,
                                     16,
                                     LR_DEFAULTCOLOR or LR_LOADREALSIZE);
      end; { with }

      InitAppClass := WordBool(RegisterClassEx(cls));
    end
  else InitAppClass := True;
end;


Function WinMain : integer;
  { application entry point                                                   }
var
  Wnd : hWnd;
  Msg : TMsg;
begin
  if not InitAppClass then Halt (255);  { register application's class        }

  { Create the main application window                                        }

  Wnd := CreateWindowEx(WS_EX_CLIENTEDGE,
                        AppName,                { class name                  }
                        AppName,                { window caption text         }
                        ws_Overlapped       or  { window style                }
                        ws_SysMenu          or
                        ws_MinimizeBox      or
                        ws_ClipSiblings     or
                        ws_ClipChildren     or  { don't affect children       }
                        ws_visible,             { make showwindow unnecessary }
                        50,                     { x pos on screen             }
                        50,                     { y pos on screen             }
                        400,                    { window width                }
                        300,                    { window height               }
                        0,                      { parent window handle        }
                        0,                      { menu handle 0 = use class   }
                        hInstance,              { instance handle             }
                        Nil);                   { parameter sent to WM_CREATE }

  if Wnd = 0 then Halt;                         { could not create the window }

  while GetMessage (Msg, 0, 0, 0) do            { wait for message            }
    begin
      TranslateMessage (Msg);                   { key conversions             }
      DispatchMessage  (Msg);                   { send to window procedure    }
    end;

  WinMain := 0;                                 { terminate with return code  }
end;

begin
  WinMain;
end.
